; -*-mode: ahk; coding:utf-8-dos-*-
;; last updated: 2012-02-07 23:49:23

;;; DvorakJ
;;; AutoHotkey_L <http://www.autohotkey.net/~Lexikos/AutoHotkey_L/>
;;; blechmusik <http://blechmusik.xii.jp/>

;;; ============================================================================
;;; ---------------- 各種設定
;;; ============================================================================

;;; Performance
;;; http://www.autohotkey.com/docs/misc/Performance.htm

#NoEnv
#SingleInstance FORCE
#WinActivateForce
; #persistent
; #Warn

SetWorkingDir %A_ScriptDir%


;;; AutoHotkey_L - #MenuMaskKey
;;; http://www.autohotkey.net/~Lexikos/AutoHotkey_L/docs/commands/_MenuMaskKey.htm
#MenuMaskKey vk07

;;; #MaxThreadsBuffer - AutoHotkeyJp
;;; http://sites.google.com/site/autohotkeyjp/reference/commands/-MaxThreadsBuffer
;;; #MaxThreadsBuffer On



#UseHook
#InstallKeybdHook
#InstallMouseHook


;;; AutoHotkeyJp: Process
;;; http://sites.google.com/site/autohotkeyjp/reference/commands/Process
Process, Priority, , High


;;; AutoHotkeyJp: SetBatchLines
;;; http://sites.google.com/site/autohotkeyjp/reference/commands/SetBatchLines
SetBatchLines, -1
;;; スクリプトの実行履歴を保存しない
ListLines Off

;;; sendmode を input に設定しない
;;; 非常に素早く何度も打鍵すると、キーフックが効かなくなることがあるから
;;; たとえば、Dvorak 配列の [H] を押したままにすると、元のキーである QWERTY 配列の [J] がときたま出力されてしまう
;;; sendmode input

;;; Console2 のようなアプリケーションでは、キーを素早く発行すると、そのキーを認識できない
;;; SetKeyDelay は、ini から設定を読み込む際に設定する
;SetKeyDelay, 0
;SetControlDelay, 0
;SetMouseDelay, 0
SetWinDelay, 0

#HotkeyInterval 1000
#MaxHotkeysPerInterval 100



DetectHiddenWindows, Off
DetectHiddenText, Off

;; タイマー割り込みを禁止する
Thread, NoTimers


;;; ==============================
;;; ==============================
;;; ==============================
;;; 設定ファイルを指定する
dvorakj_dir := A_ScriptDir
setting_file_path_folder := dvorakj_dir . to_windows_path("/user/")
setting_file_path_filename := "setting.ini"
version_ini := dvorakj_dir . to_windows_path("/data/version.ini")

;;; ==============================
;;; ==============================
;;; ==============================

;; 設定画面の処理とのやりとりに使うファイル
dvorakj_info_ini := A_Temp . to_windows_path("/dvorakj_info.ini")
;; そのファイルを新たに作成する
delete_and_append_file(dvorakj_info_ini)
;;; 現時刻を記録しておく
dvorakj_info_mtime := A_Now . A_MSec
IniWrite, %dvorakj_info_mtime% , %dvorakj_info_ini%, itself, dvorakj_info_mtime





this_key := ""

;;; 真に同時に打鍵する処理を使用するとき
key_stack := Array()




;;; ============================================================================
;;; ---------------- tray icon
;;; ============================================================================


;;; 実行ファイルのときには
;;; タスクトレイ内にアイコンを表示しない
if ( A_IsCompiled ) {
	Menu, TRAY, NoIcon
} else {
	Menu, Tray, Icon, %dvorakj_dir%\icon\folder.ico
}



;;; ============================================================================
;;; ---------------- include
;;; ============================================================================

#Include %A_ScriptDir%
;;; オブジェクト操作を拡張する
;;; スクリプトの初めの方で読み込むこと
; #Include .\src\lib\ext_object.ahk


;;; ============================================================================
;;; ---------------- ini ファイルで管理する項目
;;; ============================================================================


;;; ini ファイルの指定
ini_file := setting_file_path_folder . setting_file_path_filename
IfExist, %ini_file%
    ini_file_exists := True

;;; ini ファイルから設定を読み込む
read_general_setting()

;;; 発行するキー毎の遅延時間
;;; AutoExecute section にて設定すること
SetKeyDelay, %key_delay%


; 設定ファイルを作成した筈なので、
; 設定ファイルが存在していることにする
ini_file_exists := True

;;; ============================================================================
;;; ---------------- 設定画面を生成する
;;; ============================================================================

Gui, Add, Text, , プログラムを起動しています
Gui, Add, Progress, vlvl -Smooth w250 h18 ; PBS_MARQUEE = 0x8
Gui, Add, Text, vprogress_text w200, 1/8: 設定画面を作成中
Gui, Show, X%ProgressX% Y%ProgressY% , DvorakJ


if (A_IsCompiled)
	run, % dvorakj_dir . to_windows_path("/apps/DvorakJ_manager.exe")
else
	run, % dvorakj_dir . to_windows_path("/apps/DvorakJ_manager.ahk")


;;; ============================================================================
;;; ---------------- キーボード配列の設定を設定ファイルから読み込む
;;; ============================================================================

;;; 直接入力用配列
GuiControl, , lvl, 20
GuiControl, Text, progress_text, 2/8: 直接入力用配列を設定中
read_layout_of_English( FilePathOfUserEnglishLayout )


;;; 日本語入力用配列
GuiControl, , lvl, 30
GuiControl, Text, progress_text, 3/8: 日本語入力用配列を設定中
read_layout_of_Japanese( FilePathOfUserJapaneseLayout )


;;; ショートカットキー
GuiControl, , lvl, 40
GuiControl, Text, progress_text, 4/8: 独自のショートカットキーを設定中
; 独自のショートカットキー（無変換と変換版）

If ( is_Muhenkan_Henkan )
{
	read_layout_of_muhenkan_henkan( FilePathOfUserShortcutKeyMuhenkanHenkan )
}






;;; 無変換と変換
GuiControl, , lvl, 50
GuiControl, Text, progress_text, 5/8: 無変換＋文字と変換＋文字を設定中
;;; 独自のショートカットキー
If ( is_User_Shortcut_Key )
{
	read_layout_of_shortcutkey( FilePathOfUserShortcutKey )
}



;;; ファンクションキー
GuiControl, , lvl, 60
GuiControl, Text, progress_text, 6/8: 独自のファンクションキーを設定中
;;; 独自のファンクションキー
If ( is_User_Function_Key )
{
	read_layout_of_function_key( FilePathOfUserFunctionKey )
}





;;; テンキー
GuiControl, , lvl, 70
GuiControl, Text, progress_text, 7/8: 独自のテンキーを設定中
;;; 独自のテンキー
If ( is_User_Ten_Key )
{
	read_layout_of_ten_key( FilePathOfUserTenKey )
}




GuiControl, , lvl, 80
GuiControl, Text, progress_text, 8/8: その他


;;; ============================================================================
;;; ---------------- DvorakJ 用ホットキーを設定
;;; ============================================================================

remap_dvorakj_hotkey()

;;; ============================================================================
;;; ---------------- 更新情報を取得
;;; ============================================================================

;;; DvorakJ 起動時に更新情報を取得
if ( is_Checking_Update_On_StartUp ) {
    Run,%dvorakj_dir%\DvorakJ_Updater.exe --silent
}

;;; ============================================================================
;;; ---------------- timer
;;; ============================================================================

set_various_timers()

SetTimer, watch_dvorakj_info_ini_update, 100

GUI, Destroy

return

; dvorakj_info_ini の更新時刻を定期的に監視する
watch_dvorakj_info_ini_update:
	if ( dvorakj_info_mtime != read_value_from_dvorakj_info_ini("itself", "dvorakj_info_mtime"))
	{
		SetTimer, watch_dvorakj_info_ini_update, off

		dvorakj_info_mtime := read_value_from_dvorakj_info_ini("itself", "dvorakj_info_mtime")

		;; 終了
		if ( read_value_from_dvorakj_info_ini("state", "close")) {
			ExitApp
		}

		;; 再起動
		if ( read_value_from_dvorakj_info_ini("state", "reload")) {
			reload
		}

		;; DvorakJ の処理を停止/開始する
		switch_process := ( read_value_from_dvorakj_info_ini("state", "switch_process"))
		if (switch_process) {
			Suspend, On
		}
		else
		{
			Suspend, Off
		}


		;; 一般的な設定項目を読み込み直す
		if ( read_value_from_dvorakj_info_ini("state", "other-update")) {
			read_general_setting()
            
			write_value_to_dvorakj_info_ini("state", "other-update", False)
			;; 各タイマーを設定し直す
			set_various_timers()
		}

		;; キーボード配列の設定ファイルを読み直す
		if ( read_value_from_dvorakj_info_ini("state", "keyboard-layout-update")) {

			Gui, 9:Add, Progress, vlvl  -Smooth 0x8 w300 h18
			Gui, 9:Add, Text, , 設定ファイルを読み込みんでいます
	        Gui, 9:Show, X%ProgressX% Y%ProgressY%, DvorakJ - 再設定
	        SetTimer, Push, 45


			keyboard_layout_update_number := read_value_from_dvorakj_info_ini("state", "keyboard-layout-update")
			layout_path := read_value_from_dvorakj_info_ini("state", "keyboard-layout-path")

			if (keyboard_layout_update_number = 1)
				read_layout_of_English( layout_path )
			else
			if (keyboard_layout_update_number = 2)
				read_layout_of_Japanese( layout_path )
			else
			if (keyboard_layout_update_number = 3)
				read_layout_of_shortcutkey( layout_path )
			else
			if (keyboard_layout_update_number = 4)
				read_layout_of_muhenkan_henkan( layout_path )
			else
			if (keyboard_layout_update_number = 5)
				read_layout_of_function_key( layout_path )
			else
			if (keyboard_layout_update_number = 6)
				read_layout_of_ten_key( layout_path )
			
			write_value_to_dvorakj_info_ini("state", "keyboard-layout-update", False)
			write_value_to_dvorakj_info_ini("state", "keyboard-layout-path", False)
            
	        Gui, 9:Destroy

		}
  



		SetTimer, watch_dvorakj_info_ini_update, on
	}
return



;;; ============================================================================
;;; ---------------- キー・フック
;;; ============================================================================


remap_dvorakj_hotkey(){
    global
    static error_text
    
    If not ( is_hotkey_of_DvorakJ ) {
        return False
    }

    error_text := ""

    for i, dvorakj_hotkey in ["suspend_key", "resume_key", "toggle_key", "restart_key"]
	{
		for j, raw_v in string_to_array(%dvorakj_hotkey%, "|")
		{
		    subroutine_name := "sub_" . RTrims(dvorakj_hotkey, "_key")
            v := raw_v
            v := string_replace(v, "S-", "+")
            v := string_replace(v, "C-", "^")
            v := string_replace(v, "A-", "!")
            v := string_replace(v, "W-", "#")
            v := string_replace(v, "->", " & ")
		
		    try
		    {
		        Hotkey, %v%, %subroutine_name%
		    }
		    catch
		    {
                if ("" = error_text) {
                    error_text .= "ホットキーを設定できません。`n`n"
                    error_text .= "action`thotkey`t`n"
                    error_text .= "-----------------------`n"
                }
                
		        error_text .= RTrims(dvorakj_hotkey, "_key") "`t" raw_v "`t( " v " )`n"
		    }
		}
    }
    
    if not ("" = error_text) {
        msgbox, , ホットキーを設定できません - DvorakJ,  %error_text%
    }
    
    return True
}


sub_suspend:
	Suspend, Permit
	toggle_dummy(True)
return

sub_resume:
	Suspend, Permit
	toggle_dummy(False)
return

sub_toggle:
	Suspend, Permit
	toggle_dummy( !( A_IsSuspended ) )
return

sub_restart:
	reload
return



;;; ============================================================================
;;; ---------------- ESC, 全角, CapsLock 変換, 無変換, ひらがな
;;; ============================================================================

;;; [Space] 関連の設定
#Include .\src\keyhook\Space_Key.ahk

;;; [Shift] 関連の設定
#Include .\src\keyhook\Shift_Key.ahk

;;; [BackSpace] 関連の設定
#Include .\src\keyhook\BackSpace_Key.ahk

;;; [Enter] 関連の設定
#Include .\src\keyhook\Enter_Key.ahk

;;; [Tab] 関連の設定
#Include .\src\keyhook\Tab_Key.ahk

;;; [ESC] 関連の設定
#Include .\src\keyhook\ESC_Key.ahk

;;; [全角] 関連の設定
#Include .\src\keyhook\Zenkaku_Key.ahk

;;; [CapsLock] 関連の設定
#Include .\src\keyhook\CapsLock_Key.ahk

;;; [無変換] 関連の設定
#Include .\src\keyhook\Muhenkan_Key.ahk

;;; [変換] 関連の設定
#Include .\src\keyhook\Henkan_Key.ahk

;;; [Kana] 関連の設定
#Include .\src\keyhook\Kana_Key.ahk

;;; [Apps] 関連の設定
#Include .\src\keyhook\Apps_Key.ahk

;;; [LWin] 関連の設定
#Include .\src\keyhook\LWin_Key.ahk

;;; [RWin] 関連の設定
#Include .\src\keyhook\RWin_Key.ahk

;;; [LAlt] 関連の設定
#Include .\src\keyhook\LAlt_Key.ahk

;;; [RAlt] 関連の設定
#Include .\src\keyhook\RAlt_Key.ahk

;;; テンキーの設定
#Include .\src\keyhook\Ten_Key.ahk

;;; ファンクションキーの設定
#Include .\src\keyhook\Function_Key.ahk

;;; 通常のキーの設定
#Include .\src\keyhook\normal_Key.ahk

#Include .\src\keyhook\Get_setting_each_key.ahk

;;; キーの押し下げと押し上げを厳密に監視する
#Include .\src\keyhook\key_down_and_up.ahk


/*
 * 文字キー [\] / [_] を [Shift] のように動作させる
 * 以下のコメントアウトを解除し、以下の doubleKeys.ahk 内を適宜コメントアウトすること
 * （そうしないと、"Max Hotkeys" というエラー・メッセージが表示されて、処理が停止してしまう）
 * その上で、キーボード配列の設定で "+01E" のように組み合わせるキーを書く
 */


;;; 複数のキーの設定
#Include .\src\keyhook\multiple_Keys.ahk
;;; ホットキーを実際に押し下げているかどうかを判定する
;;; 実際には押し下げられていないとすると、他のソフトウェアがそのキーを仮想的に送信していると考えられる
#Include .\src\keyhook\get_state_of_modifier_and_key.ahk


;;; ============================================================================
;;; ---------------- [C]-[Space] などで IME の on / off を切り替える
;;; ============================================================================

#if  ( IME_toggle_key_1 )
    Ctrl & Space::
        IME_SET( !IME_GET() )
    return

#if ( IME_toggle_key_2 )
    RWin & Space::
    LWin & Space::
        IME_SET( !IME_GET() )
    return

#if ( IME_toggle_key_3 )
    Alt & Space::
        IME_SET( !IME_GET() )
    return

#if ( IME_toggle_key_4 )
    sc07b & sc079::
    sc079 & sc07b::
        IME_SET( !IME_GET() )
    return


#if

;;; ============================================================================
;;; ---------------- 状態の保持を中止
;;; ============================================================================


Enter::
    ;; vista で skkime を使うと IME の入力モードを正常に返さない不具合あり
    ;; 対処法として次の処理を考えた

    ;; skkime 使用時に Enter が押されたら
    ;; 日本語配列に切り替える
    ;; すなわち、IME を有効にする
    If ( ( Is_skkime_mode )
     and ( Is_skkime_temp_english_layout ) ) {
        nJapaneseLayoutMode := Is_skkime_temp_english_layout
        Is_skkime_temp_english_layout := 0
    }

    send( remove_sequential_strokes("{Enter}") )
return

#UseHook off




;;; ============================================================================
;;; ---------------- キーボード配列
;;; ============================================================================

#Include .\src\read\ReadLayout_Keyboard.ahk
#Include .\src\read\ReadLayout_English.ahk
#Include .\src\read\ReadLayout_Japanese.ahk
#Include .\src\read\ReadLayout_FunctionKey.ahk
#Include .\src\read\ReadLayout_MuHenkanHenkan.ahk
#Include .\src\read\ReadLayout_ShortcutKey.ahk
#Include .\src\read\ReadLayout_TenKey.ahk

;;; ============================================================================
;;; ---------------- keyhook
;;; ============================================================================

;;; 使用しないキーのフックを解除する
#Include .\src\keyhook\SetSpecialKeyHook.ahk

;;; ============================================================================
;;; ---------------- IME
;;; ============================================================================

;;; IME の状態を定期的に取得し、それに応じてキーフックの設定を切り替える処理
#Include .\src\IME\subroutine_IME_Get.ahk
;;; IME を操作する
#Include .\src\IME\IME.ahk

;;; ============================================================================
;;; ---------------- Send
;;; ============================================================================

;;; 直接入力用配列と日本語入力用配列の設定
#Include .\src\send\ReverseLetter.ahk

;;; 直接入力用配列
#Include .\src\send\subroutine_SendEnglishLayout.ahk

;;; 日本語入力用配列
#Include .\src\send\subroutine_SendJapaneseLayout.ahk

;;; 独自のショートカットキー
#Include .\src\send\send_user_shortcutkey.ahk

;;; 同時打鍵の処理
#Include .\src\send\KeyDown_KeyUp.ahk

;;; JLOD配列 のように順に打鍵する配列の設定
#Include .\src\send\send_str_by_sequential_strokes.ahk

;; 直接入力と日本語入力のどちらの入力方式で出力するかを決定する決定する
#Include .\src\send\GetOutputLang.ahk

;;; 同時打鍵の判定　タイマー入り
#Include .\src\send\outputChar.ahk


;;; ============================================================================
;;; --------------- mouse
;;; ============================================================================

;;; マウスをくるくる回すとスクロール
#Include .\src\mouse\MouseScrolling.ahk
#Include .\src\mouse\WheelScroll.ahk

;;; ============================================================================
;;; ---------------- hashing
;;; ============================================================================

;;; 素数を生成する
#Include .\src\hashing\MaxPrimeNumber.ahk
;;; 文字列を Ascii Code もどきに変換する
#Include .\src\hashing\PsuedoAsciiCode.ahk
#Include .\src\hashing\PsuedoAsciiCode_WithModifiedKeys.ahk


;;; ============================================================================
;;; ---------------- init
;;; ============================================================================

#Include .\src\init\set_various_timers.ahk
#Include .\src\init\read_setting.ahk
#Include .\src\init\io_dvorakj_ini.ahk

;;; ============================================================================
;;; ---------------- lib
;;; ============================================================================

;;; 配列を操作する
#Include .\src\lib\Array.ahk

;;; コメントアウトを除去する
#Include .\src\lib\remove_comment_out.ahk

;;; skkime 用の設定
#Include .\src\lib\SKKIME.ahk

#Include .\src\lib\ext_func_calc.ahk
#Include .\src\lib\ext_func_control.ahk
#Include .\src\lib\ext_func_file.ahk
#Include .\src\lib\ext_func_flow.ahk
#Include .\src\lib\ext_func_gui.ahk
#Include .\src\lib\ext_func_ini.ahk
#Include .\src\lib\ext_func_key.ahk
#Include .\src\lib\ext_func_memdll.ahk
#Include .\src\lib\ext_func_mouse.ahk
#Include .\src\lib\ext_func_object.ahk
#Include .\src\lib\ext_func_shell.ahk
#Include .\src\lib\ext_func_str.ahk

;;; テキストをファイルから抜き出す get_text を使用する
#Include .\src\lib\get_text.ahk



;;; ============================================================================
;;; ---------------- system
;;; ============================================================================

;;; スキャンコードを取得
#Include .\src\system\GetScanCode.ahk
;;; 設定ファイルから設定ウィンドウの表示位置の設定を読み込む
#Include .\src\system\GetWinPos.ahk
;;; フォントサイズを取得する
#Include .\src\system\GetDPI.ahk
;;; Caps Lock の値を定期的に調べる
#Include .\src\system\CapsLockState.ahk
;;; ime の情報をレジストリから定期的に調べる
#Include .\src\system\Check_ime_name.ahk
;;; 実際に押し下げている修飾キーを取得する
#Include .\src\system\get_modified_keys_pressed_down.ahk

;;; ============================================================================
;;; ---------------- str
;;; ============================================================================

;;; 出力用に文字列を変換する
#Include .\src\str\conv_str_for_sending.ahk

;;; キーボード配列の設定を整理する
#Include .\src\str\SortKCmb.ahk



toggle_dummy(bSuspended){
	global

	SetTimer, watch_dvorakj_info_ini_update, off

	write_value_to_dvorakj_info_ini("state", "bSuspended", bSuspended)
	write_value_to_dvorakj_info_ini("state", "switch_process", bSuspended)

    If ( bSuspended )
    {
	    SetTimer, IME_GET, Off
    }
    else
    {
	    SetTimer, IME_GET, On
    }


	SetTimer, watch_dvorakj_info_ini_update, on

	return
}



Push:
  GuiControl, 9:, lvl, 1
Return
